pub enum PlocSearchDistance {
Minimum,
VeryLow,
Low,
Medium,
High,
VeryHigh,
}Expand description
In PLOC, the number of nodes before and after the current one that are evaluated for pairing. Minimum (1) has a fast path in building and still results in decent quality BVHs especially when paired with a bit of reinsertion.
Variants